/**
 * Get from local storage if user is logged in, faster than waiting for API
 * when api returns, if not logged in we'll take them to login page
 */
var loggedInLocal = localStorage.getItem("isLoggedIn") !== null ? true : false;
var Auth = {
    loggedIn: loggedInLocal
};
if (loggedInLocal) {
    var bothOpts = localStorage.getItem("isLoggedIn").split(",");
    Auth.user = {};
    Auth.user.uid = bothOpts[0];
    Auth.user.email = bothOpts[1];
}
/**
 * Vue Component that handles 
 * login and sign up
 */
var LoginSignup = {
    template: "#signup-login-template",
    data: {
        auth: {},
        signUpError: ""
    },
    props: [
        "signUpError", "auth"
    ],
    methods: {
        signup: function() {
            app.signUpError = "";

            firebase.auth().createUserWithEmailAndPassword($("#signup-email").val(), $("#signup-password").val()).catch(function(error) {
                // Handle Errors here.
                app.signUpError = error.message;
            });
        },
        login: function() {
            app.signUpError = "";

            firebase.auth().setPersistence(firebase.auth.Auth.Persistence.SESSION).then(function() {
                // Existing and future Auth states are now persisted in the current
                // session only. Closing the window would clear any existing state even
                // if a user forgets to sign out.
                // ...
                // New sign-in will be persisted with session persistence.
                firebase.auth().signInWithEmailAndPassword($("#login-email").val(), $("#login-password").val()).then(function(aaa) {
                    console.log("YAY", aaa);
                    app.auth = aaa;
                }).catch(function(error) {
                    console.log("errror", error.message);
                });
            }).catch(function(error) {
                // Handle Errors here.
                // let errorCode = error.code;
                // let errorMessage = error.message;
                app.signUpError = error.message;
                console.log("errror", error.message);
            });
        },
        signout: function() {
            firebase.auth().signOut().then(function() {
                app.auth = {};
            }, function(error) {
                // An error happened.
            });
        }
    }
};
/**
 * Vue Component that handles 
 * the home page
 */
var Home = {
    template: "#home-template",
    data: function() {
        return {addEditForm: 'Add', itemsList: {}, auth: {}}
    },
    created: function(){
        this.populateItems();
        this.auth = Auth.user;
    },
    methods: {
        populateItems: function() {
            var self = this;
            db.collection("items").onSnapshot(function(snapshot) {
                snapshot.docChanges.forEach(function(change) {
                    let data = change.doc.data();
                    if (change.type === "added") {
                        console.log("ADDED: ", data);
                        var newObj = {};
                        newObj[change.doc.id] = data;
                        self.itemsList = Object.assign({}, self.itemsList, newObj);
                    }
                    if (change.type === "modified") {
                        var newObj = {};
                        newObj[change.doc.id] = data;
                        self.itemsList = Object.assign({}, self.itemsList, newObj);
                        console.log("Modified: ", data);
                    }
                    if (change.type === "removed") {
                        console.log(change.doc.id, self.itemsList);
                        delete self.itemsList[change.doc.id];
                        self.itemsList = Object.assign({}, self.itemsList, self.itemsList);
                        console.log("Removed  ", data);
                    }
                });
                console.log(self.itemsList);
            });
        },
        addItem: function(objToAdd) {
            db.collection("items").add(objToAdd).then(function(doc) {
                console.log(doc, "added!");
            }).catch(function(error) {
                console.error("Error adding document: ", error);
            });
        },
        editItem: function(id, value) {
            var self = this;
            self.itemBeingEditedId = id;
            self.addEditForm = 'Edit';
            $.each(value, function(key, value) {
                var ctrl = $('[name=' + key + ']', "#item-form");
                switch (ctrl.prop("type")) {
                    case "radio":
                    case "checkbox":
                        ctrl.each(function() {
                            if ($(this).attr('value') == value) 
                                $(this).attr("checked", value);
                            }
                        );
                        break;
                    default:
                        ctrl.val(value);
                }
            });
            UIkit.modal("#add-edit-modal").show();
        },
        updateItem: function(updateItem) {
            var self = this;
            db.collection("items").doc(self.itemBeingEditedId).set(updateItem).then(function() {
                console.log("Document successfully updated!");
            }).catch(function(error) {
                console.error("Error updating: ", error);
            });
        },
        onSubmit: function(e) {
            var self = this;
            var objToAdd = {};
            $.each($(e.target).serializeArray(), function(itemInd, itemVal) {
                objToAdd[itemVal.name] = itemVal.value;
            });
            // Move to the if below if you dont want edited items to appear at bottom
            objToAdd["date"] = Date.now();
            if (self.addEditForm === "Add") {
                self.addItem(objToAdd);
            } else {
                self.updateItem(objToAdd);
                self.addEditForm = "Add";
                self.itemBeingEditedId = false;
            }
            $(':input', e.target).not(':button, :submit, :reset, :hidden').val('').prop('checked', false).prop('selected', false);
            UIkit.modal("#add-edit-modal").hide();
        },
        deleteItem: function(itemId) {
            db.collection("items").doc(itemId).delete().then(function() {
                console.log("Document successfully deleted!");
            }).catch(function(error) {
                console.error("Error removing document: ", error);
            });
        },
        sortItems: function(items) {
            var newSorted = [];
            Object.keys(items).sort(function(a, b) {
                return items[a].date - items[b].date;
            }).forEach(function(key) {
                var newItem = {};
                newItem["id"] = key;
                newItem["value"] = items[key];
                newSorted.push(newItem);
            });
            console.log(newSorted);
            return newSorted;
        },
        signout: function() {
            var self = this;
            firebase.auth().signOut().then(function() {
                self.auth = {};
                router.push("/");
            }, function(error) {
                // An error happened.
            });
        }
    }
};
/**
 * Handling the navigation to each url
 */
const routes = [
    {
        path: '/',
        component: LoginSignup
    }, {
        path: '/login',
        component: LoginSignup
    }, {
        path: '/home',
        component: Home,
        props: true,
        /**
         * Only allow access to home page if logged in!
         */
        meta: {
            requiresAuth: true
        }
    }
];
const router = new VueRouter({
    routes // short for `routes: routes`
});
/**
 * Check if logged otherwise take to login page
 */
router.beforeEach((to, from, next) => {
    if (to.matched.some(record => record.meta.requiresAuth) && !Auth.loggedIn) {
        next({
            path: '/login',
            query: {
                redirect: to.fullPath
            }
        });
    } else {
        next();
    }
});

var app = new Vue({
    el: '#app',
    router
});

$(function() {
    app.signUpError = "";
    firebase.auth().onAuthStateChanged(function(user) {
        if (user) {
            Auth.loggedIn = true;
            Auth.user = user;
            localStorage.setItem("isLoggedIn",""+user.uid+","+user.email);
            router.push({
                path: '/home'
            });
            console.log("LOGGEDIN");
        } else {
            router.push('login');
            localStorage.removeItem("isLoggedIn");
            console.log("SIGNED OUT");
        }
    });
});
